c----------------------------------------------------------------------
c     Solves
c
c     1-Rarefaction wave for the ultrarelativistic
c     fluid in cartesian coordinates.
c     see ../doc/
c     see ../maple/
c----------------------------------------------------------------------
c     neq = 2
c----------------------------------------------------------------------

c----------------------------------------------------------------------
c     Implements differential equations.
c     Called by ODEPACK routine LSODA.
c----------------------------------------------------------------------
      subroutine fcn_rw2(neq,xi,y,yprime)

        implicit    none

        include    'fcn_rw.inc'

        character*8 cdnm
        parameter ( cdnm = 'fcn_rw2' )

        logical     ltrace
        parameter ( ltrace = .false. )

        integer     neq
        real*8      xi,    y(neq),  yprime(neq)

        real*8      PP, u
        real*8      c
        real*8      dPPdtau,   dPPdS,   dvdtau,    dvdS
        real*8      d2PPdutau, d2PPduS, d2PPd2tau, d2PPd2S, d2PPdtauS
        real*8      t0 
c======================================================================
c======================================================================
           c = sqrt(Gamma1-1.0d0)
           if ( ltrace ) then
              write(0,*) cdnm,': Gamma1=',Gamma1
              write(0,*) cdnm,':      c=',c
           endif
           call calc_prim(y(1),y(2),PP,u,Gamma1)
c-------------------------------------------------------------------
c Following expressions produced with maple. 
c-------------------------------------------------------------------

      t0 = -c**2*(1.D0+u**2)/(c**2*u**2-1.D0)
           dPPdtau   = t0 
      t0 = 2.D0/(c**2*u**2-1.D0)*u*c**2
           dPPdS     = t0 
      t0 = -u*c**2*(-1.D0+u**2)/PP/(c**2*u**2-1.D0)
           dvdtau    = t0 
      t0 = (-1.D0+u**2-c**2*u**2+c**2*u**4)*c**2/PP/(1.D0+c**2)/(c**2*u*
     #*2-1.D0)
           dvdS      = t0 
      t0 = -2.D0/(c**2*u**2-1.D0)*u*c**2+2.D0*c**4*(1.D0+u**2)/(c**2*u**
     #2-1.D0)**2.D0*u
           d2PPdutau = t0     
      t0 = -4.D0/(c**2*u**2-1.D0)**2.D0*u**2*c**4+2.D0/(c**2*u**2-1.D0)*
     #c**2
           d2PPduS   = t0     
           d2PPd2tau = d2PPdutau*dvdtau     
           d2PPd2S   = d2PPduS*dvdS    
           d2PPdtauS = d2PPdutau*dvdS     

      t0 = 1.D0/(dvdtau-u*dvdtau*dPPdS+(1.D0-u**2)*d2PPdtauS/2.D0-1.D0/s
     #qrt((-2.D0*u-(1.D0-u**2)*dPPdS)**2.D0-4.D0*u**2+4.D0*(1.D0-u**2)*d
     #PPdtau)*(2.D0*(-2.D0*u-(1.D0-u**2)*dPPdS)*(-2.D0*dvdtau+2.D0*u*dvd
     #tau*dPPdS-(1.D0-u**2)*d2PPdtauS)-8.D0*u*dvdtau-8.D0*u*dvdtau*dPPdt
     #au+4.D0*(1.D0-u**2)*d2PPd2tau)/4.D0+(dvdS-u*dvdS*dPPdS+(1.D0-u**2)
     #*d2PPd2S/2.D0-1.D0/sqrt((-2.D0*u-(1.D0-u**2)*dPPdS)**2.D0-4.D0*u**
     #2+4.D0*(1.D0-u**2)*dPPdtau)*(2.D0*(-2.D0*u-(1.D0-u**2)*dPPdS)*(-2.
     #D0*dvdS+2.D0*u*dvdS*dPPdS-(1.D0-u**2)*d2PPd2S)-8.D0*u*dvdS-8.D0*u*
     #dvdS*dPPdtau+4.D0*(1.D0-u**2)*d2PPdtauS)/4.D0)*(u+(1.D0-u**2)*dPPd
     #S/2.D0-sqrt((-2.D0*u-(1.D0-u**2)*dPPdS)**2.D0-4.D0*u**2+4.D0*(1.D0
     #-u**2)*dPPdtau)/2.D0))
           yprime(1) = t0

      t0 = (u+(1.D0-u**2)*dPPdS/2.D0-sqrt((-2.D0*u-(1.D0-u**2)*dPPdS)**2
     #.D0-4.D0*u**2+4.D0*(1.D0-u**2)*dPPdtau)/2.D0)/(dvdtau-u*dvdtau*dPP
     #dS+(1.D0-u**2)*d2PPdtauS/2.D0-1.D0/sqrt((-2.D0*u-(1.D0-u**2)*dPPdS
     #)**2.D0-4.D0*u**2+4.D0*(1.D0-u**2)*dPPdtau)*(2.D0*(-2.D0*u-(1.D0-u
     #**2)*dPPdS)*(-2.D0*dvdtau+2.D0*u*dvdtau*dPPdS-(1.D0-u**2)*d2PPdtau
     #S)-8.D0*u*dvdtau-8.D0*u*dvdtau*dPPdtau+4.D0*(1.D0-u**2)*d2PPd2tau)
     #/4.D0+(dvdS-u*dvdS*dPPdS+(1.D0-u**2)*d2PPd2S/2.D0-1.D0/sqrt((-2.D0
     #*u-(1.D0-u**2)*dPPdS)**2.D0-4.D0*u**2+4.D0*(1.D0-u**2)*dPPdtau)*(2
     #.D0*(-2.D0*u-(1.D0-u**2)*dPPdS)*(-2.D0*dvdS+2.D0*u*dvdS*dPPdS-(1.D
     #0-u**2)*d2PPd2S)-8.D0*u*dvdS-8.D0*u*dvdS*dPPdtau+4.D0*(1.D0-u**2)*
     #d2PPdtauS)/4.D0)*(u+(1.D0-u**2)*dPPdS/2.D0-sqrt((-2.D0*u-(1.D0-u**
     #2)*dPPdS)**2.D0-4.D0*u**2+4.D0*(1.D0-u**2)*dPPdtau)/2.D0))
           yprime(2) = t0
           if ( ltrace ) then 
              write(0,*) cdnm,': yprime(1)=',yprime(1)      
              write(0,*) cdnm,': yprime(2)=',yprime(2)    
              write(0,*) cdnm,'::::::::::::::::::::::::::'    
           endif
        return
      end
